@@ -2,8 +2,6 @@ package X3D;
 use strict;
 use warnings;
 
-our $VERSION = 0.01;
-
 use rlib 'X3D';
 
 use X3DBrowser;
@@ -29,11 +27,7 @@ X3D - X3D Developer Toolkits
 	
 	my $scene = Browser->createX3DFromString("Group{}");
 
-	print $scene;
-	
-	Browser->replaceWorld( $scene );
-
-	Browser->startTime(time);
+	printf $scene;
 
 =head1 DESCRIPTION
 
