@@ -7,7 +7,7 @@ use rlib "../../../";
 use Gtk2;
 use Glib qw(TRUE FALSE);
 
-use X3DObject;
+use Array;
 use X3DError;
 
 # maybe bad style, but makes life a lot easier
@@ -15,7 +15,7 @@ use base qw(Exporter);
 
 our @EXPORT = qw(
   COL_IS_NODE
-  COL_NODEID
+  COL_NODE
   COL_IS_FIELD
   COL_FIELD
   COL_ROUTE
@@ -24,7 +24,7 @@ our @EXPORT = qw(
 
 use enum qw(
   COL_IS_NODE
-  COL_NODEID
+  COL_NODE
   COL_IS_FIELD
   COL_FIELD
   COL_ROUTE
@@ -54,13 +54,13 @@ sub INIT_INSTANCE {
 	$this->{n_columns}    = N_COLUMNS;
 	$this->{column_types} = [
 		'Glib::Boolean',    # COL_IS_NODE
-		'Glib::String',     # COL_NODEID
+		'Glib::Scalar',     # COL_NODE
 		'Glib::Boolean',    # COL_IS_FIELD
 		'Glib::Scalar',     # COL_FIELD
-		'Glib::String',     # COL_ROUTE
+		'Glib::Uint',       # COL_ROUTE
 	];
 
-	$this->{root}   = undef;
+	$this->{root}  = undef;
 	$this->{record} = {};
 
 	# Random int to check whether an iter belongs to our model
@@ -120,10 +120,7 @@ sub GET_COLUMN_TYPE {
 
 sub GET_ITER {
 	my ( $this, $path ) = @_;
-	# print #####################################################################
-	X3DError::Debug "*" x 3;
-	X3DError::Debug $path->to_string;
-	#############################################################################
+	#X3DError::Debug $path->to_string;
 
 	my @indices = $path->get_indices;
 	my $depth   = $path->get_depth;
@@ -132,74 +129,27 @@ sub GET_ITER {
 
 	my $n      = 0;
 	my $object = $this->{root};
-	my $parent;
 
 	foreach (@indices) {
-		$n      = $_;
-		$parent = $object;
-
+		$n = $_;
 		if ( $object->isa('SFNode') ) {
-
-			$object = SFNodeGetNTHField( $object, $n );
-			return unless ref $object;
-
+			my $node      = $object->getValue;
+			my $fieldName = $node->getFieldDefinitions->[$n]->getName;
+			$object = $node->getField($fieldName);
 		} elsif ( $object->isa('X3DField') ) {
-
 			if ( $object->getType eq "MFNode" ) {
-
-				$object = FieldGetNTHSFNode( $object, $n );
-				return unless ref $object;
-
-			} elsif ( $object->getType eq "SFNode" ) {
-
 				$object = $object->getValue;
-				return unless ref $object;
-
-			} else {
-				die;
+				# Is this the last record in the list?
+				return undef if $n > $#$object;
+				$object = $object->[$n];
 			}
-
-		} else {
-			die;
 		}
 	}
 
 	return undef unless ref $object;
 
-	# print #####################################################################
-	X3DError::Debug $n, "object: ", ref $object;
-	X3DError::Debug $n, "parent: ", ref $parent;
-	X3DError::Debug $n, $object->getValue->getName, $object->getValue->getType if $object->isa('SFNode');
-	X3DError::Debug $n, "o type: ", $object->getType, $object->getName if $object->isa('X3DField');
-	#############################################################################
-
-	# debug #####################################################################
-	die if $object->isa('X3DField') && $parent->isa('X3DField');
-	#############################################################################
-
-	return $this->ITER( $n, $object, $parent );
-}
-
-sub SFNodeGetNTHField {
-	my $sfnode = shift;
-	my $n      = shift;
-
-	my $node             = $sfnode->getValue;
-	my $fieldDefinitions = $node->getFieldDefinitions;
-	return undef if $n > $#$fieldDefinitions;
-
-	my $fieldName = $fieldDefinitions->[$n]->getName;
-	return $node->getField($fieldName);
-}
-
-sub FieldGetNTHSFNode {
-	my $field  = shift;
-	my $n      = shift;
-	my $mfnode = $field->getValue;
-	# Is this the last record in the list?
-	return undef if $n > $#$mfnode;
-
-	return $mfnode->[$n];
+	#X3DError::Debug $n, ref $parent, ref $object->getValue;
+	return $this->ITER( $n, object => $object );
 }
 
 #
@@ -226,35 +176,26 @@ sub GET_PATH {
 
 sub GET_VALUE {
 	my ( $this, $iter, $column ) = @_;
-	# print #####################################################################
 	#X3DError::Debug @$iter;
-	#############################################################################
 
 	die "bad iter" unless $iter;
 
 	return undef unless $column < @{ $this->{column_types} };
 
-	my $object = $iter->[2]->{object};
 	my $parent = $iter->[2]->{parent};
+	my $object = $iter->[2]->{object};
 
 	return undef unless ref $object;
 
 	if ( $object->isa("SFNode") ) {
-
-		return TRUE                     if $column == COL_IS_NODE;
-		return $object->getValue->getId if $column == COL_NODEID;
-
-		return
-		  sprintf "%s %s %d",
-		  ref $object,
-		  ref $parent,
-		  Hash::NoRef::SvREFCNT($object)
-		  if $column == COL_ROUTE;
+		return TRUE    if $column == COL_IS_NODE;
+		return $object if $column == COL_NODE;
+		return 1       if $column == COL_ROUTE;
 
 	} elsif ( $object->isa("X3DField") ) {
-		return TRUE           if $column == COL_IS_FIELD;
-		return $object        if $column == COL_FIELD;
-		return $object->getId if $column == COL_ROUTE;
+		return TRUE    if $column == COL_IS_FIELD;
+		return $object if $column == COL_FIELD;
+		return 2       if $column == COL_ROUTE;
 	}
 
 	return;
@@ -266,49 +207,35 @@ sub GET_VALUE {
 
 sub ITER_NEXT {
 	my ( $this, $iter ) = @_;
+	#X3DError::Debug $iter->[1], ref $iter->[2]->{parent}, ref $iter->[2]->{object};
+
+	return undef unless $iter && $iter->[2]->{parent};
 
 	my $n      = $iter->[1] + 1;
+	my $parent = $iter->[2]->{object}->getParents;
 	my $object = $iter->[2]->{object};
-	my $parent = $iter->[2]->{parent};
 
-	# print #####################################################################
-	X3DError::Debug "*" x 3;
-	X3DError::Debug $n, "object: ", ref $object;
-	X3DError::Debug $n, "parent: ", ref $parent;
-	X3DError::Debug $n, $object->getValue->getName, $object->getValue->getType if $object->isa('SFNode');
-	X3DError::Debug $n, "o type: ", $object->getType, $object->getName if $object->isa('X3DField');
-	#############################################################################
+	#X3DError::Debug ref $parent;
+	if ( $parent->isa("SFNode") ) {
+		my $node             = $parent->getValue;
+		my $fieldDefinitions = $node->getFieldDefinitions;
+		return undef if $n > $#$fieldDefinitions;
 
-	if ( $object->isa("SFNode") ) {
+		my $fieldName = $node->getFieldDefinitions->[$n]->getName;
+		$object = $node->getField($fieldName);
+	} elsif ( $parent->isa("X3DField") ) {
 		if ( $parent->getType eq "MFNode" ) {
-			$object = FieldGetNTHSFNode( $parent, $n );
-			return unless ref $object;
-		} elsif ( $parent->getType eq "SFNode" ) {
-			return;
-		} else {
-			die;
+			$object = $parent->getValue;
+			# Is this the last record in the list?
+			return undef if $n > $#$object;
+			$object = $object->[$n];
 		}
-
-	} elsif ( $object->isa("X3DField") ) {
-		$object = SFNodeGetNTHField( $parent, $n );
-		return unless ref $object;
 	} else {
 		die;
 	}
 
-	# print #####################################################################
-	X3DError::Debug;
-	X3DError::Debug $n, ref $object;
-	X3DError::Debug $n, $object->getValue->getName, $object->getValue->getType if $object->isa('SFNode');
-	X3DError::Debug $n, $object->getType, $object->getName if $object->isa('X3DField');
-	#############################################################################
-
-	# debug #####################################################################
-	die if $object->isa('X3DField') && $parent->isa('X3DField');
-	die unless ref $object;
-	#############################################################################
-
-	return $this->ITER( $n, $object, $parent );
+	#X3DError::Debug $iter->[1];
+	return $this->ITER( $n, object => $object );
 }
 
 #
@@ -340,39 +267,31 @@ sub ITER_HAS_CHILD { &ITER_N_CHILDREN(@_) }
 
 sub ITER_N_CHILDREN {
 	my ( $this, $iter ) = @_;
-
-	X3DError::Debug( ref $iter ? 1 : 0 );
+	#X3DError::Debug $iter->[2];
 
 	# special case: if iter == NULL, return number of top-level rows
-	return $this->{root}->getValue->length if !$iter;
+	return $this->{root}->length if !$iter;
 
-	my $n      = 0;
+	my $n      = $iter->[1];
+	my $parent = $iter->[2]->{parent};
 	my $object = $iter->[2]->{object};
 
-	# print #####################################################################
-	X3DError::Debug "*" x 3;
-	X3DError::Debug ref $object;
-	#############################################################################
-
-	if ( $object->isa("SFNode") ) {
+	if ( $object->isa("MFNode") ) {
+		die;
+	} elsif ( $object->isa("SFNode") ) {
 		my $node             = $object->getValue;
 		my $fieldDefinitions = $node->getFieldDefinitions;
 		#X3DError::Debug scalar @$fieldDefinitions;
-		$n = @$fieldDefinitions;
+		return scalar @$fieldDefinitions;
 	} elsif ( $object->isa("X3DField") ) {
 		$object = $object->getValue;
 
-		$n = $object->length if $object->isa("MFNode");
-		$n = $object ? 1 : 0 if $object->isa("SFNode");
-	} else {
-		die;
+		return $object->length if $object->isa("MFNode");
+		#X3DError::Debug ref $object;
 	}
 
-	# print #####################################################################
-	X3DError::Debug $n;
-	#############################################################################
-
-	return $n;
+	# no children zB: SFBool, SFInt32
+	return 0;
 }
 
 #
@@ -385,51 +304,41 @@ sub ITER_N_CHILDREN {
 
 sub ITER_NTH_CHILD {
 	my ( $this, $iter, $n ) = @_;
+	#X3DError::Debug $n;
 
-	# iter == NULL is a special case; we need to return the first top-level row
 	my $object;
-	my $parent = $iter ? $iter->[2]->{object} : $this->{root};
-
-	# print #####################################################################
-	X3DError::Debug "*" x 3;
-	X3DError::Debug $n, ref $parent;
-	X3DError::Debug $n, $parent->getValue->getName, $parent->getValue->getType if $parent->isa('SFNode');
-	X3DError::Debug $n, $parent->getType, $parent->getName if $parent->isa('X3DField');
-	#############################################################################
-
-	if ( $parent->isa("SFNode") ) {
-
-		$object = SFNodeGetNTHField( $parent, $n );
-
-	} elsif ( $parent->isa("X3DField") ) {
-
-		if ( $parent->getValue->isa("MFNode") ) {
-
-			$object = FieldGetNTHSFNode( $parent, $n );
-
-		} elsif ( $parent->getValue->isa("SFNode") ) {
-
-			$object = $parent->getValue;
-
-		} else {
-			die;
-		}
+	my $parent;
 
+	if ($iter) {
+		$object = $iter->[2]->{object};
+		$parent = $iter->[2]->{parent};
 	} else {
-		die;
+		# iter == NULL is a special case; we need to return the first top-level row
+		return undef unless ref $this->{root};
+		$object = $this->{root};
+	}
+
+	if ( $object->isa("SFNode") ) {
+		my $node             = $object->getValue;
+		my $fieldDefinitions = $node->getFieldDefinitions;
+		return undef if $n > $#$fieldDefinitions;
+
+		my $fieldName = $fieldDefinitions->[$n]->getName;
+		$parent = $object;
+		$object = $node->getField($fieldName);
+	} elsif ( $object->isa("X3DField") ) {
+		$parent = $object;
+		$object = $object->getValue;
+
+		if ( $object->isa("MFNode") ) {
+			return undef if $n > $#$object;
+			$object = $object->[$n];
+		}
+		#X3DError::Debug "X3DField", ref $object;
 	}
 
-	# print #####################################################################
-	X3DError::Debug $n, ref $object;
 	#X3DError::Debug $n, ref $parent, ref $object;
-
-	# debug #####################################################################
-	die if $object->isa('X3DField') && $parent->isa('X3DField');
-	#############################################################################
-
-	return unless ref $object;
-
-	return $this->ITER( $n, $object, $parent );
+	return $this->ITER( $n, parent => $parent, object => $object );
 }
 
 #
@@ -440,139 +349,44 @@ sub ITER_NTH_CHILD {
 
 sub ITER_PARENT {
 	my ( $this, $iter ) = @_;
+	X3DError::Debug ref $iter->[2]->{parent}, ref $iter->[2]->{object};
 
 	my $n;
-	my $object = $iter->[2]->{parent};
 	my $parent;
+	my $object = $iter->[2]->{parent};
 
-	# print #####################################################################
-	X3DError::Debug "*" x 3;
-	X3DError::Debug "object: ", ref $object;
-	X3DError::Debug "parent: ", ref $parent;
-	X3DError::Debug "  root: ", ref $this->{root};
-	X3DError::Debug $object->getValue->getName, $object->getValue->getType if $object->isa('SFNode');
-	X3DError::Debug "o type: ", $object->getType, $object->getName if $object->isa('X3DField');
-	#############################################################################
-
+	return undef unless ref $object;
 	return if $object == $this->{root};
-	X3DError::Debug;
 
 	if ( $object->isa("SFNode") ) {
-
-		( $n, $parent ) = SFNodeGetParentField($object);
-		X3DError::Debug ">", $n, ref $parent;
-
-	} elsif ( $object->isa("X3DField") ) {
-
-		( $n, $parent ) = FieldGetParentSFNode($object);
-		X3DError::Debug ">", $n, ref $parent;
-		#die;
-		# 		$object = $object->getParents;
-		#
-		# 		$parent = $object->getParents;
-		# 		if ( $parent->isa("SFNode") ) {
-		# 			$n = Array::index( [ $parent->getFieldDefinitions ], $object );
-		# 		} elsif ( $parent->isa("MFNode") ) {
-		# 			$n = $parent->index($object);
-		# 		}
-		#
-		# 		X3DError::Debug $parent, $object;
-	} else {
-		X3DError::Debug "else 2", ref $object;
-		die;
-	}
-
-	# print #####################################################################
-	X3DError::Debug $n, ref $object;
-	X3DError::Debug $n, $object->getValue->getName, $object->getValue->getType if $object->isa('SFNode');
-	X3DError::Debug $n, $object->getType, $object->getName if $object->isa('X3DField');
-	#############################################################################
-
-	# debug #####################################################################
-	die if $object->isa('X3DField') && $parent->isa('X3DField');
-	die if $object->isa('SFNode')   && $parent->isa('SFNode');
-	return die unless ref $parent;
-	#############################################################################
-
-	return $this->ITER( $n, $object, $parent );
-}
-
-sub SFNodeGetParentField {
-	my $sfnode = shift;
-	return NodeGetParentField( $sfnode->getValue );
-}
-
-sub NodeGetParentField {
-	my $child_node = shift;
-
-	my ($sfnode) = $child_node->getParents;
-	my $node = $sfnode->getValue;
-
-	foreach my $field ( $node->getFields(TRUE) ) {
-		if ( $field->getType eq "SFNode" ) {
-
-			X3DError::Debug "sf", ref $field;
-			return ( 0, $field )
-			  if $field->getValue->getValue == $child_node;
-
-		} elsif ( $field->getType eq "MFNode" ) {
-
-			my $n = 0;
-			foreach ( @{ $field->getValue } ) {
-				X3DError::Debug "mf", ref $field;
-				return ( $n, $field ) if $_->getValue == $child_node;
-				++$n;
-			}
-
+		$object = $object->getParents;
+		
+		$parent = $object->getParents;
+		if ( $parent->isa("SFNode")) {
+			$n = Array::index([$parent->getFieldDefinitions], $object);
+		} elsif ( $parent->isa("MFNode")) {
+			$n = $parent->index($object);
 		}
-	}
-
-	X3DError::Debug "no parent field";
-	return ();
-}
-
-sub FieldGetParentSFNode {
-	my $field = shift;
-
-	# n #########################################################################
-
-	my $fieldDefinitions = $field->getParent->getValue->getFieldDefinitions;
-
-	my $n = 0;
-	foreach (@$fieldDefinitions) {
-		last if $_->getName eq $field->getName;
-		++$n;
-	}
-
-	return () if $n > $#$fieldDefinitions;
-
-	# sfnode ####################################################################
-
-	my ($node) = $field->getParent->getValue;
-	my ( $i, $parent_field ) = NodeGetParentField($node);
-	my $sfnode;
-
-	if ( $parent_field->getValue->isa("SFNode") ) {
-
-		$sfnode = $parent_field->getValue;
-
-	} elsif ( $parent_field->getValue->isa("MFNode") ) {
-
-		my $index = $parent_field->getValue->getIndex($node);
-		return () if $index == -1;
-		$sfnode = $parent_field->getValue->[$index];
 
+		X3DError::Debug $parent, $object;
+		die;
+	} elsif ( $object->isa("X3DField") ) {
+		X3DError::Debug "X3DField", ref $object;
+		#X3DError::Debug ref $iter->[2]->{parent}, ref $iter->[2]->{object};
+		die;
 	} else {
+		X3DError::Debug "else", ref $object;
+	
 		die;
 	}
 
-	return ( $n, $sfnode );
+	X3DError::Debug $n, ref $parent, ref $object;
+	return $this->ITER( $n, parent => $parent, object => $object );
 }
 
 sub ITER {
-	my ( $this, $n, $object, $parent ) = @_;
-	$this->{record}->{object} = $object;
-	$this->{record}->{parent} = $parent;
+	my ( $this, $n, @values ) = @_;
+	%{ $this->{record} } = @values;
 	return [ $this->{stamp}, $n, $this->{record} ];
 }
 
@@ -581,34 +395,8 @@ sub ITER {
 # of nodes in the model.  you normally don't need to do anything for these,
 # but may want to if you plan to implement data caching.
 #
-#sub REF_NODE { X3DError::Debug "REF_NODE @_\n"; }
-#sub UNREF_NODE {
-#	my ( $this, $iter ) = @_;
-#	X3DError::Debug "UNREF_NODE @_";
-#}
-
-############################################################################
-############################################################################
-############################################################################
-
-sub set_children {
-	X3DError::Debug;
-	my ( $this, $field ) = @_;
-	$this->{root} = $field;
-	my $children = $field->getValue;
-
-	X3DError::Debug 0, "...", $#$children;
-	foreach ( 0 ... $#$children ) {
-		X3DError::Debug $_, ref $children->[$_];
-	}
-
-	foreach ( 0 ... $#$children ) {
-		my $path = new Gtk2::TreePath($_);
-		my $iter = $this->get_iter($path);
-		$this->row_inserted( $path, $iter );
-		$this->row_has_child_toggled( $path, $iter );
-	}
-}
+#sub REF_NODE { X3DError::Debug; X3DError::Debug "REF_NODE @_\n"; }
+#sub UNREF_NODE { X3DError::Debug; X3DError::Debug "UNREF_NODE @_\n"; }
 
 1;
 __END__
