@@ -4,8 +4,8 @@ use warnings;
 
 use rlib "./";
 
-use MFScalar;
-use base qw(MFScalar);
+use MFValue;
+use base qw(MFValue);
 
 use SFImage;
 our $SField = new SFImage;
