@@ -4,8 +4,8 @@ use warnings;
 
 use rlib "./";
 
-use MFScalar;
-use base qw(MFScalar);
+use MFValue;
+use base qw(MFValue);
 
 use SFNode;
 our $SField = new SFNode;
@@ -32,3 +32,15 @@ sub toString {
 
 1;
 __END__
+sub addParents {
+	my ( $this, @parents ) = @_;
+	$_->addParents(@parents) foreach @$this;
+	return;
+}
+
+sub removeParents {
+	my ( $this, @parents ) = @_;
+	$_->addParents(@parents) foreach @$this;
+	return;
+}
+
