@@ -2,9 +2,9 @@ package X3DFieldDefinition;
 use strict;
 use warnings;
 
-use base qw(X3DObject);
+#use rlib "./";
 
-use X3DField;
+use base qw(X3DField);
 
 sub create {
 	my $this = shift;
@@ -12,14 +12,12 @@ sub create {
 }
 
 sub getField {
-	my ( $this, $sfnode ) = @_;
-	my $field = new X3DField( $sfnode, $this->{accessType}, $this->{name}, $this->{value}->copy );
+	my ( $this, $parent ) = @_;
+	my $field = new X3DField( $this->{accessType}, $this->{name}, $this->{value}->copy );
+	$field->addParents($parent);
 	return $field;
 }
 
-sub getAccessType { $_[0]->{accessType} }
-sub getValue      { $_[0]->{value} }
-
 sub toString {
 	my $this   = shift;
 	my $string = "";
